# Please e-mail mathatfa@gmail.com about bugs in this script.
# 

#require("export")
#require("modelr")
require("tidyverse")
require("tidybayes")
#require("broom")
require("openxlsx")
# require("ggstance")
# require("ggridges")
# require("cowplot")
require("rstan")
require("brms")
#require("emmeans")

rstan_options(auto_write = TRUE)
cores <- min(4,parallel::detectCores()-1) # 4 chains will NOT sample faster >4 cores
options(mc.cores = cores)
theme_set(theme_light())
options(width = 120)
S_width <- 13.33/2.0
S_height <- 7.5/2

# setwd("/home/dqc/R/ReverseDesign/HarmonizedData Nabe RD 2019-01-23/")
# setwd("/home/drs/R/Nabe-Rat-CCI-flexion/EMG-M.Zhang/HarmonizedData Nabe RD 2019-01-23/")
setwd("C:/DaniilSarkisyan/R/Nabe-Rat-CCI-flexion/EMG-M.Zhang/HarmonizedData Nabe RD 2019-01-23/")
load(file="EMG-RD2019-01-23-data.RData")
setwd("../figs-RD9/")


# Which "AI of Thresholds" model is better by LOO? ----
fName <- "AI_RD9_2019-01-31.RData"
fit <- list()
fit.kfold <- list()
mybrm <- function(ModName="mymod", ...) {
  tryCatch({
    fit[[ModName]] <<- brm(
      ..., iter=40000, cores=cores, chains=cores, seed = 20190131, # thin = 4,
      save_all_pars = TRUE, sample_prior = TRUE, 
      control=list(max_treedepth=13, adapt_delta=0.999))
    print(summary(fit[[ModName]]))
    save(fit, fit.kfold, file=fName)
  }, error=function(e){})
  print(paste( ifelse(is.null(fit[[ModName]]), "Failed:", "Saved:"), ModName ))
}

tmp <- emgLat %>% 
  mutate( MSL = paste(Muscle, StimLocat, sep="."),
          MSL = factor(MSL, sort(unique(MSL))),
          Op3 = paste0(OperationSide,Operation),
          Op3 = replace(Op3, Operation=="Sham", "Sham"),
          Op3 = factor(Op3, c("Sham","LeftTBI","RightTBI"), c("SO","LTBI","RTBI")),
          RC.log2 = log2(RecCurrent) ) %>%
  droplevels(.)
xtabs(~Operation+MSL+Drug, data=tmp)
xtabs(~Op3+MSL, data=tmp)
xtabs(~Operation+MSL+OperationSide, data=tmp)


# AI_0_70ms, AI_0.2_1s, AI_1_10s ~ Operation*Muscle + RC.log2*Muscle + (1||RatNo) ----
mybrm(ModName=paste("AI","2","fast",sep="."),
      AI_0_70ms ~ Operation*Muscle + RC.log2*Muscle + (1||RatNo),
      data=tmp, family = student() )
mybrm(ModName=paste("AI","2","slow",sep="."),
      AI_0.2_1s ~ Operation*Muscle + RC.log2*Muscle + (1||RatNo),
      data=tmp, family = student() )
mybrm(ModName=paste("AI","2","after1",sep="."),
      AI_1_10s ~ Operation*Muscle + RC.log2*Muscle + (1||RatNo),
      data=tmp, family = student() )

# AI_0_70ms, AI_0.2_1s, AI_1_10s ~ (SO vs LTBI vs RTBI)*Muscle + RC.log2*Muscle + (1||RatNo) ----
mybrm(ModName=paste("AI","3","fast",sep="."),
      AI_0_70ms ~ Op3*Muscle + RC.log2*Muscle + (1||RatNo),
      data=tmp, family = student() )
mybrm(ModName=paste("AI","3","slow",sep="."),
      AI_0.2_1s ~ Op3*Muscle + RC.log2*Muscle + (1||RatNo),
      data=tmp, family = student() )
mybrm(ModName=paste("AI","3","after1",sep="."),
      AI_1_10s ~ Op3*Muscle + RC.log2*Muscle + (1||RatNo),
      data=tmp, family = student() )


# AI_0_70ms, AI_0.2_1s, AI_1_10s ~ Operation*OperationSide*Muscle + RC.log2*Muscle + (1||RatNo) ----
mybrm(ModName=paste("AI","4","fast",sep="."),
      AI_0_70ms ~ Operation*OperationSide*Muscle + RC.log2*Muscle + (1||RatNo),
      data=tmp, family = student() )
mybrm(ModName=paste("AI","4","slow",sep="."),
      AI_0.2_1s ~ Operation*OperationSide*Muscle + RC.log2*Muscle + (1||RatNo),
      data=tmp, family = student() )
mybrm(ModName=paste("AI","4","after1",sep="."),
      AI_1_10s ~ Operation*OperationSide*Muscle + RC.log2*Muscle + (1||RatNo),
      data=tmp, family = student() )


# compare all models ----
#load(file=fName)

for(m in names(fit)) {
  if(!is.null(fit.kfold[[m]])) next
  fit.kfold[[m]] <- kfold( fit[[m]] )
  fit.kfold[[m]]$model_name <- m
  print(paste("Validated:", m))
  save(fit, fit.kfold, file=fName)
}
#compare_ic(x = fit.kfold, ic = "kfold")
names(fit)
